/*****************************
* Source : XbZLXipClass.prg
* System : <unkown>
* Author : Phil Ide
* Created: 10/12/2003
*
* Purpose:
* ----------------------------
* History:
* ----------------------------
*    10/12/2003 13:00 PPI - Created
*****************************/

#include "Common.ch"
#include "XbZLib.ch"
#include "fileio.ch"
#include "directry.ch"

#define CLASS_NAME XbZlXip

CLASS CLASS_NAME
   PROTECTED:
      VAR aFStruct
      VAR cZip
      VAR nDir

      METHOD ok

   EXPORTED:
      VAR zipFile
      VAR compression

      METHOD init

      METHOD Close
      METHOD Open
      METHOD addFile

ENDCLASS

METHOD CLASS_NAME:init()
   ::zipFile := 0
   ::compression := Z_BEST_COMPRESSION
   ::aFStruct := {}
   
   return self

METHOD CLASS_NAME:Open( cPath, nAppend )
   default nAppend to APPEND_STATUS_CREATE

   if nAppend == APPEND_STATUS_CREATE
      ::zipFile := FCreate( cPath )
   elseif nAppend == APPEND_STATUS_ADDINZIP
      // not currently supported
   endif
   ::cZip := cPath
   return self

METHOD CLASS_NAME:ok()
   return ::zipFile > 0

METHOD CLASS_NAME:Close()
   local cRec
   local v
   local nCRecSize := 0
   local nCDirOff := FSeek(::zipFile, 0, FS_RELATIVE )

   if ::ok()
      FWrite(::zipFile, CENTRAL_DIR_SIGNATURE )
      For i := 1 to Len(::aFStruct)
         cRec := U2Bin(::aFStruct[i]:offset)
         cRec += ::aFStruct[i]:fileTimeDate
         cRec += ::aFStruct[i]:attrib
         cRec += ::aFStruct[i]:file

         v := Len(cRec)
         cRec := I2Bin(v)+cRec

         nCRecSize += Len(cRec)

         FWrite(::zipFile,cRec)
      Next
      // now write end-of-central-dir record
      FWrite(::zipFile, END_OF_CENTRAL_DIR_RECORD+U2Bin(nCDirOff)+U2Bin(Len(::aFStruct))+U2Bin(nCRecSize) )
      FClose(::zipFile)
      ::zipFile := 0
   endif
   return self

// central dir structure:
// [signature]
// [record1]
// ...
// [recordN]
// [end-of-central-dir record]
//
// record structure:
// ULONG size_of_remainder_of_record
// ULONG offset_to_start_of_local_record
// ULONG file_time_and_date_in_DOS_format
// ULONG file_attributes
// CHAR* remainder of record is filename
//
// end-of-central-dir record structure
// ULONG signature
// ULONG offset to start of central directory
// ULONG number of entries in central directory
// ULONG size (in bytes) of central directory
//
// local record structure
// ULONG signature
// UINT  size of remainder of record
// ULONG uncomp file size
// ULONG comp file size
// ULONG crc32 of uncompressed file
// UINT  comp mode (0=stored, 8=deflate)
// CHAR* remainder of record is filename

METHOD CLASS_NAME:addFile(cFile)
   local cRec
   local aFile
   local cData
   local nErr := -1
   local crc
   local cBuff
   local o
   local cFAttr
   local x := 0
   local nComp := 8

   if ::ok() .and. FExists(cFile)
      aFile := Directory(cFile,"D")

      cBuff := MemoRead(cFile)
      /*
      crc := xbzCrc32(0, cBuff, Len(cBuff))
      if crc < 1
         crc := Abs(crc)
      endif
      */
      crc := crc32(cBuff)

      cData := xbzCompress( cBuff, @nErr )
      if Len(cData) == Len(cBuff)
         cData := cBuff
         nComp := 0
      endif


      if nErr == 0
         o := xbxip_InternalFile():new(cFile)
         aadd(::aFStruct, o)
         o:fileTimeDate := Xbzfiletime(cFile, @cFAttr)
         o:attrib := cFAttr
         o:offset := FSeek(::zipFile, 0, FS_RELATIVE)

         cRec := U2Bin( aFile[1][F_SIZE] )
         cRec += U2Bin( Len( cData ) )
         cRec += crc // U2Bin(crc)
         cRec += I2Bin(nComp)
         cRec += cFile

         cRec := LOCAL_FILE_RECORD+I2Bin(Len(cRec))+cRec
         FWrite(::zipFile,cRec)
         FWrite(::zipFile, cData)
      endif
   endif
   return nErr









