/*****************************
* Source : XbZLZipClass.prg
* System : <unkown>
* Author : Phil Ide
* Created: 10/12/2003
*
* Purpose:
* ----------------------------
* History:
* ----------------------------
*    10/12/2003 12:45 PPI - Created
*****************************/

#include "Common.ch"
#include "XbZLib.ch"
#include "directry.ch"
#include "fileio.ch"

CLASS xbZlZip
   PROTECTED:
      VAR aFStruct
      VAR nDir
      VAR nMode

      METHOD loadDirectory
      METHOD loadCDEntry
      METHOD acceptFromZip

   EXPORTED:
      VAR zipFile READONLY
      VAR compression
      VAR fileName READONLY
      VAR gComment

      METHOD init

      METHOD Close
      METHOD Open
      METHOD addFile
      METHOD directory
      METHOD extract
      METHOD extractAll
      METHOD moveFromZip
      METHOD moveToZip
      METHOD ok

ENDCLASS

METHOD xbZlZip:init()
   ::zipFile := 0
   ::compression := Z_DEFAULT_COMPRESSION
   ::aFStruct := {}
   ::fileName := ''
   ::nMode := -1
   ::gComment := ''
   return self

METHOD xbZlZip:Open( cPath, nAppend )
   default nAppend TO ZLZIP_OPEN_CREATE

   do case
      case nAppend == ZLZIP_OPEN_CREATE
         ::zipFile := FCreate( cPath )

      case nAppend == ZLZIP_OPEN_APPEND .and. FExists(cPath)
         ::zipFile := FOpen( cPath, FO_READWRITE )

      case nAppend == ZLZIP_OPEN_READ .and. FExists(cPath)
         ::zipFile := FOpen( cPath )


   endcase
   if ::zipFile > 0
      ::nMode := nAppend
      if ::nMode > ZLZIP_OPEN_CREATE
         ::directory()
      endif
   endif
   ::fileName := iif (::ok( nAppend < ZLZIP_OPEN_READ ), cPath, '')

   return self

METHOD xbZlZip:Close(globalComment)
   local i
   local oECDRec
   local nSize := 0
   local cData
   local nPos

   default globalComment to ::gComment

   if ::ok(TRUE)
      nPos := FSeek(::zipFile, 0, FS_RELATIVE )
      for i := 1 to Len(::aFStruct)
         cData := ::aFStruct[i]:get()
         FWrite(::zipFile, cData)
         nSize += Len(cData)
      next
      // write end of central dir record
      oECDRec := ZipEndOfCentralDirectoryRecord():new()

      oECDRec.signature       := END_OF_CENTRAL_DIR_RECORD
      oECDRec.diskNumber      := 0
      oECDRec.diskNumWStartCD := 0
      oECDRec.entriesThisDisk := Len(::aFStruct)
      oECDRec.entries         := Len(::aFStruct)
      oECDRec.sizeOfCD        := nSize
      oECDRec.startOfCD       := nPos
      oECDRec.commentLen      := Len(globalComment)
      oECDRec.comment         := globalComment

      FWrite(::zipFile, oECDRec:get())
      FClose(::zipFile)
      ::zipFile := 0
   elseif ::ok()
      FClose(::zipFile)
   endif
   return self

METHOD xbZlZip:ok(lWrite)
   default lWrite TO FALSE
   return ::zipFile > 0 .and. iif( lWrite, ::nMode < ZLZIP_OPEN_READ, TRUE )


METHOD xbZlZip:addFile( cFile, cRootDir, cComment )
   local crc
   local nUNCFSize
   local cFAttr
   local aF
   local lOk := FALSE
   local cData
   local nErr := 0
   local offset
   local nIAF
   local cBuff
   local oLFRec := ZipLocalFileRecord():new()
   local oCDRec := ZipCentralDirectoryRecord():new()
   local cSpare
   local cFFile

   default cComment to '',;
           cRootDir to ''

   if !Empty(cRootDir) .and. !(cRootDir[-1] = '\' .or. cRootDir[-1] == '/')
      cRootDir += '\'
   endif

   if ::ok(TRUE) .and. FExists(cRootDir+cFile)
      aF := Directory(cRootDir+cFile)
      aadd( ::aFStruct, oCDRec )
      offSet := FSeek(::zipFile, 0, FS_RELATIVE )

      cBuff := MemoRead(cRootDir+cFile)
      nIAF := if( at(chr(0),cBuff) > 0, 0, 1 )

      crc := crc32(@cBuff)
      nUNCFSize := aF[1][F_SIZE]
      cData := xbzCompress( cBuff, @nErr )

      cSpare := Left(cData,2)
      cData := SubStr(cData,3)

      if Len(cData) == nUNCFSize
         nCompMeth := 0
      endif

      cFFile := NormaliseFileName(cFile)

      oLFRec.signature        := LOCAL_FILE_RECORD
      oLFRec.verToExtract     := 0x14
      oLFRec.gpFlag           := 0
      oLFRec.compMethod       := iif( Len(cData) == nUNCFSize, 0, 8 )
      oLFRec.fileTime         := Xbzfiletime(cRootDir+cFile, @cFAttr)
      oLFRec.crc              := crc
      oLFRec.compSize         := Len(cData)
      oLFRec.unCompSize       := nUNCFSize
      oLFRec.fNameLen         := Len(cFFile)
      oLFRec.extraFldLen      := 0
      oLFRec.fileName         := cFFile
      oLFRec.extraFld         := ''

      oCDRec.signature        := CENTRAL_DIR_SIGNATURE
      oCDRec.verMadeBy        := 0x14
      oCDRec.VerToExtract     := 0x14
      oCDRec.gpFlag           := 0
      oCDRec.compMethod       := oLFRec:compMethod
      oCDRec.fileTime         := oLFRec.fileTime
      oCDRec.crc              := crc
      oCDRec.compSize         := oLFRec.compSize
      oCDRec.unCompSize       := nUNCFSize
      oCDRec.fNameLen         := Len(cFFile)
      oCDRec.extraFldLen      := 0
      oCDRec.commentLen       := Len(cComment)
      oCDRec.diskNumStart     := 0
      oCDRec.internalFAttrib  := nIAF
      oCDRec.externalFAttrib  := cFAttr
      oCDRec.offset           := offSet
      oCDRec.fileName         := cFFile
      oCDRec.extraFld         := ''
      oCDRec.comment          := cComment

      FWrite(::zipFile, oLFRec:get())
      FWrite(::zipFile, cData)

      lOk := TRUE
   endif
   return self

METHOD xbZlZip:directory() //cMask, lExact, lCaseSensitive, lRegExp)
   local aRet := {}

   if ::ok() .and. ::nMode == ZLZIP_OPEN_READ
      if Empty(::aFStruct)
         ::loadDirectory()
      endif
   endif
   return ::aFStruct

METHOD xbZlZip:loadDirectory()
   local oECDRec := ZipEndOfCentralDirectoryRecord():new()
   local cBuff := Space(oECDRec:sizeOf()+0xffff)
   local nECDREcPos := Len(cBuff)
   local i, n
   local nOffset

   nECDREcPos := FSeek(::zipFile, -Len(cBuff), FS_END )
   n := FRead(::zipFile, @cBuff, Len(cBuff) )
   cBuff := Left( cBuff, n )

   i := AT( END_OF_CENTRAL_DIR_RECORD, cBuff )

   if i > 0
      nECDREcPos += (i-1) //Len(cBuff)
      cBuff := SubStr( cBuff, i )
      n := oECDRec:sizeOf()+1
      ::gComment := SubStr( cBuff, oECDRec:sizeOf()+1 )//, oECDRec.commentLen )
      nOffset := Bin2U( SubStr( cBuff, 17, 4 ) )
      FSeek(::zipFile, nOffset) // go to first central dir record

      i := 0
      While (n := FSeek(::zipFile, 0, FS_RELATIVE)) < nECDREcPos
         i++
         aadd(::aFStruct, ::loadCDEntry())
      Enddo
   endif
   return self

METHOD xbZlZip:loadCDEntry()
   local oCDRec := ZipCentralDirectoryRecord():new()
   local cBuff := Space( oCDRec:sizeOf() )

   FRead( ::zipFile, @cBuff, Len(cBuff) )
   oCDRec:put(cBuff)

   // get filename
   cBuff := Space(oCDRec.fNameLen)
   FRead(::zipFile, @cBuff, Len(cBuff) )
   oCDRec.filename := cBuff

   // get extra-fld
   if oCDRec.extraFldLen > 0
      cBuff := Space(oCDRec.extraFldLen)
      FRead(::zipFile, @cBuff, Len(cBuff) )
      oCDRec.extraFld := cBuff
   endif

   // get comment
   if oCDRec.commentLen > 0
      cBuff := Space(oCDRec.commentLen)
      FRead(::zipFile, @cBuff, Len(cBuff) )
      oCDRec.comment := cBuff
   endif

   oCDRec.cargo := self

   return oCDRec
// 0x9c78
METHOD xbZlZip:extract(o, cTargetRoot)
   local nOfe  := o:get()
   local nOff  := o.offset
   local nSize := o.compSize
   local cFT   := o.fileTime
   local cData := Space(nSize)
   local nHnd
   local nOk := 0
   local oFRec := ZipLocalFileRecord():new()
   local cBuff

   default cTargetRoot to '.\'

   if o.compMethod == ZLZIP_COMP_STORE .or. o.compMethod == ZLZIP_COMP_DEFLATED
      cData := Space(oFRec:sizeOf())
      FSeek( ::zipFile, nOff, FS_SET )
      FRead( ::zipFile, @cData, Len(cData) )
      oFRec:put(cData)
      FSeek(::zipFile, oFRec.fNameLen+oFRec.extraFldLen, FS_RELATIVE)
      nHnd := FSeek(::zipFile, 0, FS_RELATIVE)
      cData := Space(nSize)
      FRead( ::zipFile, @cData, nSize )

      if Empty(cBuff)
         cBuff := xbzUnCompress( ZLIB_SIGNATURE+cData, o.unCompSize )
      endif

      nHnd := FCreate( cTargetRoot+o.fileName )
      if nHnd > 0
         FWrite( nHnd, cBuff )
         FClose(nHnd)
         nOk := 1
      endif
   else
      nOk := -o.compMethod
   endif
   return nOk

METHOD xbZlZip:extractAll(cTargetRoot)
   local i
   local n := 0

   for i := 1 to Len(::aFStruct)
      n += iif( ::extract(::aFStruct[i],cTargetRoot) > 0, 1, 0 )
   next
   return n

METHOD xbZlZip:moveFromZip( o )
   return o:cargo:moveToZip( o, self )

METHOD xbZlZip:moveToZip(o, oZip)
   local oLFRec := ZipLocalFileRecord():new()
   local nOff := o.offset
   local cData
   lOk := FALSE

   if ::ok() .and. oZip:ok(lWrite)
      // firstly, gather the local-file record and the data
      FSeek(::zipFile, nOff, FS_SET)
      cData := Space( oLFRec:sizeOf() )
      FRead( ::zipFile, @cData, Len(cData) )
      oLFRec:put( cData )

      cData := Space( oLFRec.fNameLen + oLFRec.extraFldLen )
      FRead( ::zipFile, @cData, Len(cData) )

      oLFRec.fileName := Left( cData, oLFRec.fNameLen )
      oLFRec.extraFld := Right( cData, oLFRec.extraFldLen )

      cData := Space( oLFRec.compSize )
      FRead( ::zipFile, @cData, Len(cData) )

      // ok, now insert the data into the new file
      lOk := oZip:acceptFromZip( oLFRec, o, cData )
   endif
   return lOk


METHOD xbZlZip:acceptFromZip( oLFRec, oXCDRec, cData )
   local oCDRec := OClone( oXCDRec )

   oCDRec.diskNumStart     := 0
   oCDRec.offset           := FSeek(::zipFile, 0, FS_RELATIVE )
   oCDRec.cargo            := self

   FWrite(::zipFile, oLFRec:get() )
   FWrite(::zipFile,  cData, Len(cData) )
   aadd(::aFiles, oCDRec)
   return TRUE

STATIC Function NormaliseFileName(cFile)
   cFile := StrTran(cFile,'\','/')
   return cFile

