/**********************************************
 SOURCE : myArrayBrowse.prg
 SYSTEM : generic
 AUTHOR : Phil Ide
 PURPOSE: MyArrayBrowse CLASS (XbpBrowse() wrapper)

 Copyright (c) Phil Ide 2001, All Rights Reserved
 Copyright (c) Alex Blankendaal 2001, All Rights Reserved

 NOTES:
 This is a modified version of my MyBrowse() class, which
 performs similar functionality for an array browse.

 To use:
   oBrowse := myArrayBrowse():new(...) // usual XbpBrowse() params
   oBrowse:setArray(aMyArray)
   etc.

   To add columns, you can do this:

   o:addColumnBlock( nIndx, nWidth, cHeading, cFooting, nType )

   Note that this is identical to the standard o:addColumn(), except
   that instead of passing a code-block as the first parameter, you just
   pass an index into the array to identify which column of the array is
   to be associated with this column of the browse.

   an easy example:
   for i := 1 to len(oBrowse:nArray)
     oBrowse:addColumnBlock( i, len(oBrowse:nArray[1][i]), 'Col #'+LTrimStr(i)))
   next


 The code for array handlng was kindly donated by
 Alex Blankendaal, who cribbed it from the Samples directory.
 So I guess thanx also to Alaska :-)

**********************************************/

#include "Gra.ch"
#include "Xbp.ch"
#include "Common.ch"
#include "Appevent.ch"
#include "Font.ch"

class myArrayBrowse from xbpbrowse
   exported:
      var aArray
      var nPointer

      var whileclause
      var cChild

      method init
      method setup
      method winBrowseSkip
      method create
      method setArray
      METHOD addColumnBlock
endclass

method myArrayBrowse:init( oParent, oOwner, aPos, aSize, aPresParm, lVisible )
   ::xbpbrowse:init( oParent, oOwner, aPos, aSize, aPresParm, lVisible )
   ::whileclause := {|| TRUE }
   ::setup()
   return (self)

method myArrayBrowse:create( oParent, oOwner, aPos, aSize, aPresParm, lVisible )
   ::xbpbrowse:create()
   return (self)

method myArrayBrowse:setArray( aArray )
   ::aArray := aArray
   return (self)

method myArrayBrowse:setup()
   ::cursorMode := XBPBRW_CURSOR_ROW
   ::softTrack := TRUE
   ::setFontCompoundName(FONT_DEFFIXED_SMALL)
   ::hScroll := FALSE
   ::itemSelected := {|| NIL }
   ::nPointer := 1

   // Navigation code blocks for the browser
   ::skipBlock     := {|n| ::winBrowseSkip(n,::nPointer,::aArray) }
   ::goTopBlock    := {| | ::nPointer := 1      }
   ::goBottomBlock := {| | ::nPointer := len(::aArray)   }

   ::phyPosBlock   := {| | ::nPointer        }

   // Navigation code blocks for the vertical scroll bar
   ::posBlock      := {| | ::nPointer    }
   ::lastPosBlock  := {| | len(::aArray)             }
   ::firstPosBlock := {| | 1               }

   return (self)

METHOD myArrayBrowse:addColumnBlock( nIndx, nWidth, cHeading, cFooting, nType )
   local bLink := {|| Var2Char(::aArray[::nPointer, nIndx]) }
   ::addColumn( bLink, nWidth, cHeading, cFooting, nType )
   return (self)

METHOD myArrayBrowse:winBrowseSkip( nSkip ) //, nBrowseRecno, aArray )
   local nCanSkip
   local nLastRec := Len(::aArray)

   default ::nPointer to 1

   if ::nPointer + nSkip < 1               // "BoF"
      nCanSkip := 1 - ::nPointer
   elseif ::nPointer + nSkip > nLastRec  // "EoF"
      nCanSkip := nLastRec - ::nPointer
   else
      nCanSkip := nSkip
   endif

   ::nPointer += nCanSkip

   return nCanSkip

